@@ -274,6 +274,7 @@ typedef struct {
 
 	gdouble            opacity;
 	cairo_pattern_t   *opacity_mask;
+	gboolean           pop_resource_dict;
 } GXPSCanvas;
 
 static GXPSCanvas *
@@ -286,6 +287,7 @@ gxps_canvas_new (GXPSRenderContext *ctx)
 
 	/* Default values */
 	canvas->opacity = 1.0;
+	canvas->pop_resource_dict = FALSE;
 
 	return canvas;
 }
@@ -320,6 +322,22 @@ canvas_start_element (GMarkupParseContext  *context,
 
 		brush = gxps_brush_new (canvas->ctx);
 		gxps_brush_parser_push (context, brush);
+	} else if (strcmp (element_name, "Canvas.Resources") == 0) {
+		GXPSResources *resources;
+
+		if (canvas->pop_resource_dict) {
+			gxps_parse_error (context,
+					  canvas->ctx->page->priv->source,
+					  G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+					  element_name, NULL, NULL, error);
+			return;
+		}
+
+		resources = gxps_archive_get_resources (canvas->ctx->page->priv->zip);
+		gxps_resources_push_dict (resources);
+		canvas->pop_resource_dict = TRUE;
+		gxps_resources_parser_push (context, resources,
+		                            canvas->ctx->page->priv->source);
 	} else {
 		render_start_element (context,
 				      element_name,
@@ -357,6 +375,8 @@ canvas_end_element (GMarkupParseContext  *context,
 			cairo_push_group (canvas->ctx->cr);
 		}
 		gxps_brush_free (brush);
+	} else if (strcmp (element_name, "Canvas.Resources") == 0) {
+		gxps_resources_parser_pop (context);
 	} else {
 		render_end_element (context,
 				    element_name,
@@ -382,6 +402,95 @@ static GMarkupParser canvas_parser = {
 	canvas_error
 };
 
+static void
+resource_start_element (GMarkupParseContext  *context,
+                        const gchar          *element_name,
+                        const gchar         **names,
+                        const gchar         **values,
+                        gpointer              user_data,
+                        GError              **error)
+{
+	if (strcmp (element_name, "PathGeometry") == 0) {
+		GXPSPath *path = (GXPSPath *)user_data;
+
+		gxps_path_parser_push (context, path);
+	} else if (g_str_has_suffix (element_name, "Brush")) {
+		GXPSPath *path = (GXPSPath *)user_data;
+		GXPSBrush *brush;
+
+		brush = gxps_brush_new (path->ctx);
+		gxps_brush_parser_push (context, brush);
+	}
+}
+
+static void
+resource_end_element (GMarkupParseContext  *context,
+		      const gchar          *element_name,
+		      gpointer              user_data,
+		      GError              **error)
+{
+	if (strcmp (element_name, "PathGeometry") == 0) {
+		g_markup_parse_context_pop (context);
+	} else if (g_str_has_suffix (element_name, "Brush")) {
+		GXPSPath *path = (GXPSPath *)user_data;
+		GXPSBrush *brush = g_markup_parse_context_pop (context);
+
+		path->fill_pattern = cairo_pattern_reference (brush->pattern);
+		gxps_brush_free (brush);
+	}
+}
+
+static GMarkupParser resource_parser = {
+	resource_start_element,
+	resource_end_element,
+	NULL,
+	NULL,
+	NULL
+};
+
+static gboolean
+expand_resource (GXPSPage    *page,
+                 const gchar *data,
+                 gpointer     user_data)
+{
+	gchar *resource_key;
+	gchar *p;
+	gsize len;
+	GXPSResources *resources;
+	const gchar *resource;
+	GMarkupParseContext *context;
+	gboolean ret = TRUE;
+
+	if (!g_str_has_prefix (data, "{StaticResource "))
+		return FALSE;
+
+	p = strstr (data, "}");
+	if (p == NULL)
+		return FALSE;
+
+	len = strlen ("{StaticResource ");
+	resource_key = g_strndup (data + len, p - (data + len));
+
+	if (!resource_key || *resource_key == '\0') {
+		g_free (resource_key);
+		return FALSE;
+	}
+
+	resources = gxps_archive_get_resources (page->priv->zip);
+	resource = gxps_resources_get_resource (resources, resource_key);
+	g_free (resource_key);
+	if (!resource)
+		return FALSE;
+
+	context = g_markup_parse_context_new (&resource_parser, 0, user_data, NULL);
+
+	ret = g_markup_parse_context_parse (context, resource, strlen (resource), NULL) &&
+	      g_markup_parse_context_end_parse (context, NULL);
+	g_markup_parse_context_free (context);
+
+	return ret;
+}
+
 static void
 render_start_element (GMarkupParseContext  *context,
 		      const gchar          *element_name,
@@ -402,7 +511,15 @@ render_start_element (GMarkupParseContext  *context,
 		path = gxps_path_new (ctx);
 
 		for (i = 0; names[i] != NULL; i++) {
-			if (strcmp (names[i], "Data") == 0) {
+			/* FIXME: if the resource gets expanded, that specific
+			 * resource will be already handled leading to a different
+			 * behavior of what we are actually doing without resources.
+			 * In an ideal world we would handle the resource without
+			 * special casing
+			 */
+			if (expand_resource (ctx->page, values[i], path)) {
+				GXPS_DEBUG (g_message ("expanded resource: %s", names[i]));
+			} else if (strcmp (names[i], "Data") == 0) {
 				path->data = g_strdup (values[i]);
 			} else if (strcmp (names[i], "RenderTransform") == 0) {
 				cairo_matrix_t matrix;
@@ -423,8 +540,7 @@ render_start_element (GMarkupParseContext  *context,
 			} else if (strcmp (names[i], "Clip") == 0) {
 				path->clip_data = g_strdup (values[i]);
 			} else if (strcmp (names[i], "Fill") == 0) {
-				GXPS_DEBUG (g_message ("set_fill_pattern (solid)"));
-                                if (!gxps_brush_solid_color_parse (values[i], ctx->page->priv->zip, 1., &path->fill_pattern)) {
+				if (!gxps_brush_solid_color_parse (values[i], ctx->page->priv->zip, 1., &path->fill_pattern)) {
 					gxps_parse_error (context,
 							  ctx->page->priv->source,
 							  G_MARKUP_ERROR_INVALID_CONTENT,
@@ -432,6 +548,7 @@ render_start_element (GMarkupParseContext  *context,
 					gxps_path_free (path);
 					return;
 				}
+				GXPS_DEBUG (g_message ("set_fill_pattern (solid)"));
 			} else if (strcmp (names[i], "Stroke") == 0) {
 				GXPS_DEBUG (g_message ("set_stroke_pattern (solid)"));
                                 if (!gxps_brush_solid_color_parse (values[i], ctx->page->priv->zip, 1., &path->stroke_pattern)) {
@@ -711,6 +828,12 @@ render_start_element (GMarkupParseContext  *context,
 		if (canvas->opacity != 1.0)
 			cairo_push_group (canvas->ctx->cr);
 		g_markup_parse_context_push (context, &canvas_parser, canvas);
+	} else if (strcmp (element_name, "FixedPage.Resources") == 0) {
+		GXPSResources *resources;
+
+		resources = gxps_archive_get_resources (ctx->page->priv->zip);
+		gxps_resources_parser_push (context, resources,
+		                            ctx->page->priv->source);
 	} else if (strcmp (element_name, "FixedPage") == 0) {
 		/* Do Nothing */
 	} else {
@@ -775,12 +898,14 @@ render_end_element (GMarkupParseContext  *context,
 
 		if (path->opacity_mask) {
 			gdouble x1 = 0, y1 = 0, x2 = 0, y2 = 0;
+			cairo_path_t *cairo_path;
+
 			if (path->stroke_pattern)
 				cairo_stroke_extents (ctx->cr, &x1, &y1, &x2, &y2);
 			else if (path->fill_pattern)
 				cairo_fill_extents (ctx->cr, &x1, &y1, &x2, &y2);
 
-			cairo_path_t *cairo_path = cairo_copy_path (ctx->cr);
+			cairo_path = cairo_copy_path (ctx->cr);
 			cairo_new_path (ctx->cr);
 			cairo_rectangle (ctx->cr, x1, y1, x2 - x1, y2 - y1);
 			cairo_clip (ctx->cr);
@@ -952,7 +1077,15 @@ render_end_element (GMarkupParseContext  *context,
 		}
 		cairo_restore (ctx->cr);
 		GXPS_DEBUG (g_message ("restore"));
+		if (canvas->pop_resource_dict) {
+			GXPSResources *resources;
+
+			resources = gxps_archive_get_resources (ctx->page->priv->zip);
+			gxps_resources_pop_dict (resources);
+		}
 		gxps_canvas_free (canvas);
+	} else if (strcmp (element_name, "FixedPage.Resources") == 0) {
+		gxps_resources_parser_pop (context);
 	} else if (strcmp (element_name, "FixedPage") == 0) {
 		/* Do Nothing */
 	} else {
@@ -1513,41 +1646,14 @@ gxps_page_finalize (GObject *object)
 {
 	GXPSPage *page = GXPS_PAGE (object);
 
-	if (page->priv->zip) {
-		g_object_unref (page->priv->zip);
-		page->priv->zip = NULL;
-	}
-
-	if (page->priv->source) {
-		g_free (page->priv->source);
-		page->priv->source = NULL;
-	}
-
-	if (page->priv->init_error) {
-		g_error_free (page->priv->init_error);
-		page->priv->init_error = NULL;
-	}
-
-	if (page->priv->lang) {
-		g_free (page->priv->lang);
-		page->priv->lang = NULL;
-	}
-
-	if (page->priv->name) {
-		g_free (page->priv->name);
-		page->priv->name = NULL;
-	}
-
-	if (page->priv->image_cache) {
-		g_hash_table_destroy (page->priv->image_cache);
-		page->priv->image_cache = NULL;
-	}
-
-	if (page->priv->anchors) {
-		g_hash_table_destroy (page->priv->anchors);
-		page->priv->anchors = NULL;
-		page->priv->has_anchors = FALSE;
-	}
+	g_clear_object (&page->priv->zip);
+	g_clear_pointer (&page->priv->source, g_free);
+	g_clear_error (&page->priv->init_error);
+	g_clear_pointer (&page->priv->lang, g_free);
+	g_clear_pointer (&page->priv->name, g_free);
+	g_clear_pointer (&page->priv->image_cache, g_hash_table_destroy);
+	g_clear_pointer (&page->priv->anchors, g_hash_table_destroy);
+	page->priv->has_anchors = FALSE;
 
 	G_OBJECT_CLASS (gxps_page_parent_class)->finalize (object);
 }
